/*
 * Copyright (c) 2004, Jeremy Rayner
 * All rights reserved.
 */
package org.javanicus.easy;


import org.picocontainer.*;
import org.picocontainer.defaults.*;


/**
 * Simple EasyContainer implementation wrapping pico for delegation.
 * <br>
 * example usage:<br>
 * <code>
 * EasyContainer pico = new EasyPicoContainer(new DefaultPicoContainer());<br>
 * pico.reg(Girl.class);<br>
 * pico.reg(Kitten.class);<br>
 * </code>
 */
public class EasyPicoContainer implements EasyContainer {
    public EasyPicoContainer(MutablePicoContainer pico) {
        this.pico = pico;
    }
    
    private MutablePicoContainer pico;
    
    public Object get(Object key) throws PicoException {
        return pico.getComponentInstance(key);
    }
    
    public Object reg(Class implementation) throws PicoRegistrationException {
        return pico.registerComponentImplementation(implementation);
    }
    
    public Object reg(Object key, Class implementation, Class param1) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ComponentParameter(param1)});
    }

    public Object reg(Object key, Class implementation, String param1) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ConstantParameter(param1)});
    }
    
    public Object reg(Object key, Class implementation, Class param1, Class param2) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ComponentParameter(param1),
            new ComponentParameter(param2)});
    }

    public Object reg(Object key, Class implementation, Class param1, String param2) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ComponentParameter(param1),
            new ConstantParameter(param2)});
    }

    public Object reg(Object key, Class implementation, String param1, Class param2) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ConstantParameter(param1),
            new ComponentParameter(param2)});
    }

    public Object reg(Object key, Class implementation, String param1, String param2) throws PicoRegistrationException {
        return pico.registerComponentImplementation(key, implementation,
                new Parameter[] {new ConstantParameter(param1),
            new ConstantParameter(param2)});
    }
    
    public Object unreg(Object key) {
        return pico.unregisterComponent(key);
    }
    
    /**
     * Gets hold of the underlying <code>MutablePicoContainer</code> that this wraps.
     * So if you want anything not provided by this simple interface, you can still access the underlying pico API.
     * 
     * @return the wrapped picocontainer
     */
    public MutablePicoContainer getPico() {
        return pico;
    }
}
