/*
 * Copyright (c) 2004, Jeremy Rayner
 * All rights reserved.
 */
package org.javanicus.easy;


import junit.framework.*;
import org.picocontainer.defaults.DefaultPicoContainer;


public class EasyPicoContainerTest extends TestCase {
    
    protected void setUp() {
        pico = new EasyPicoContainer(new DefaultPicoContainer());
    }

    public void testCreationOfABoyUsingAnEasyContainer() {
        pico.reg(Boy.class);
        assertNotNull("boy should exist", pico.get(Boy.class));
    }
    
    public void testCreatingAGirlAndAKittenAndGettingThemToInteractWithEachOther() {
        pico.reg(Kitten.class);
        pico.reg(Girl.class);
        Girl girl = (Girl) pico.get(Girl.class);

        assertNotNull("girl should exist", girl);
        assertEquals("when I kiss my thingmy it says 'Purr'", girl.doYourThing());
    }
    
    public void testGivingTheGirlAChoiceOfAKittenAndABoyToKissWhichShouldResultInAnAmbiguousSituationForHer() {
        Girl girl = null;

        try {
            pico.reg(Kitten.class);
            pico.reg(Boy.class);
            pico.reg(Girl.class);
            girl = (Girl) pico.get(Girl.class);
            fail("girl shouldn't know whether to kiss the boy or the kitten, given the choice");
        } catch (Exception e) {
            assertNull("girl should not be created", girl);
        }
    }
    
    public void testBuildingComponentsWithParametersInTheirConstructorAndReRegistrationOfTheComponents() {
        Girl girl;

        pico.reg(Girl.class);
        pico.reg(FashionVictim.class, FashionVictim.class, "goth");
        girl = (Girl) pico.get(Girl.class);
        assertEquals("when I kiss my thingmy it says 'Kissing you back in a goth style.'",
                girl.doYourThing());
        // try reregistration of one component
        pico.unreg(FashionVictim.class);
        pico.reg(FashionVictim.class, FashionVictim.class, "metalhead");
        girl = (Girl) pico.get(Girl.class);
        assertEquals("when I kiss my thingmy it says 'Kissing you back in a goth style.'",
                girl.doYourThing());
        // try reregistration of both components
        pico.unreg(FashionVictim.class);
        pico.unreg(Girl.class);
        pico.reg(Girl.class);
        pico.reg(FashionVictim.class, FashionVictim.class, "metalhead");
        girl = (Girl) pico.get(Girl.class);
        assertEquals("when I kiss my thingmy it says 'Kissing you back in a metalhead style.'",
                girl.doYourThing());
    }
    
    private EasyContainer pico;
    
    public EasyPicoContainerTest(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(EasyPicoContainerTest.class);
    }

    public void testInstatiation() {
        assertNotNull("pico should exist", pico);
    }
    
    protected void tearDown() {
        pico = null;
    }

    /** interaces/classes for tests */    
    interface Kissable {
        String kiss();
    }


    public static class Girl {
        private Kissable thingmy;
        public Girl(Kissable kissable) {
            thingmy = kissable;
        }
        
        public String doYourThing() {
            return "when I kiss my thingmy it says '" + thingmy.kiss() + "'";
        }
    }
    

    public static class Boy implements Kissable {
        public Boy() {} 

        public String kiss() {
            return "Where's your interface?";
        }
    }
    

    public static class Kitten implements Kissable {
        public Kitten() {}

        public String kiss() {
            return "Purr";
        }
    }
    

    public static class Granny implements Kissable {
        public Granny() {}

        public String kiss() {
            return "Would you like another cake, dear?";
        }
    }


    public static class FashionVictim implements Kissable {
        public FashionVictim(String style) {
            this.style = style;
        }
        private String style;
        public String kiss() {
            return "Kissing you back in a " + style + " style.";
        }
    }
}
